:_RAM													:

.include "RAM.asm"

:_HEADER												:
	
	:_Offset_Header:	; Header start at  : PRG_size - Header_size = Offset	
	.org $7ff0		; Header start at  : $8000    - $000f       = $7ff0	

	:_Header_Data:          ;flags	; 16 byte .NES header (iNES format)	
	.byte "NES", $1a	; 0 > 3	; ASCII NES code			
	.byte $02              	; 4	; size of PRG ROM in 16 ko units	
	.byte $04		; 5	; size of CHR ROM in  8 ko units	
	.byte $00	;MMC1	; 6	; mapper X in high part = #%xxxx0000	
	.byte $00		; 7	; mapper over 15			
        .byte $00             	; 8	; size of PRG RAM in 8kb RAM		
        .byte $01		; 9 	;flags 9 : 0 = NTSC, 1 = PAL		
        .byte $00		; 10
        .byte $00
        .byte $00
        .byte $00
        .byte $00
        .byte $00
	
	:_Start_PRG:		; Program Rom start at $8000 and ending at $FFFF (size = 8000)	
	.org $8000		; PRG start at : $8000						

:_RESET     												:

	:_ClearRegisters:	; reset routine			
        sei			; set interrupt disable		
        cld			; clear decimal mode		
	ldx #$00		; load 00			
	stx $2000		; reset register PPU 1		
	stx $2001		; reset register PPU 2		
	dex			; x-1 = FF			
	txs			; reset stack register =$FF	
  	ldx #0			; clear x = 0			
  	ldy #0			; clear x = 0			
  	txa			; reset a = 0			

	:_ClearMemory:
	sta 0, X		; store at $000	+ $X the value of A = 0	
	sta $100, X		; store at $100	+ $X the value of A = 0	
	sta $200, X		; ect..					
	sta $300, X		; 					
	sta $400, X		; 					
	sta $500, X		; finally we reset the memory to 0	
	sta $600, X		; ($0 to $ 1000)			
	sta $700, X		;					
	sta $800, X		;					
	sta $900, X		;					
        inx			; X change from $00 to $FF before Z = 1	
	bne _ClearMemory:	; if Z=0 goto ClearMemory		
	
	:_InitMapper:
	.include "Mapper.asm"
	
:_INITIALISATIONS											:

	:_@VBlankWait:      	; wait for Vblank, PPU is ready after this		
	bit PPU_STATUS		; read bit 7 of PPU_STATUS				
	bpl _@VBlankWait	; if  N = 0 (bit 7 of PPU_STATUS) goto VblankWait	
		
		
	:_SetPalette:

	:SetPaletteAddress:	; palette start always at $3F00 in PPU Memory
	lda #$3F
	sta PPU_ADDR		; write low  byte
	lda #$00	
	sta PPU_ADDR		; write high byte
		
	:LoadPalette:
	ldx #(128-32)		;32 bytes =  4 x 4 sprites_colors + 4 x 4 nametable_colors
	-
	lda Palette-(128-32),x
	sta PPU_DATA		
	inx			; loop from x = 96 to 128
	bpl -			; while x < 128 branch
		
	:_ClearNameTables:
	
	:SetFirstNametable:	
	lda #$20
	sta Address_NameTable+1
	lda #$00
	sta Address_NameTable

	_SetNametableAdressing
	lda Address_NameTable+1
	sta PPU_ADDR		; $2006
	lda Address_NameTable
	sta PPU_ADDR		; $2006
	
		
	:ClearNameTable:	; screen is 32 row x 30 lines	
	ldx #240		; (32 x 30) / 4 = 960/4 = 240	
	-
	lda ClearCHRByte	; written byte
	sta PPU_DATA		; write in Nametable with PPU_DATA $2007
	sta PPU_DATA
	sta PPU_DATA
	sta PPU_DATA
	dex
	bne -			; x not=0 goto ClearLine

	:ClearSecondNametable:
	lda Address_NameTable+1
	eor #$04			;$20 eor $04 = $24
	sta Address_NameTable+1
	cmp #$20			; first time is fault
	bne _SetNametableAdressing
	
	; :_SetRightPPUMode:
	; lda Data_PPU_CTRL
	; and DisableNMI
	; and IncrementPerColumn
	; sta PPU_CTRL
	

	:_Test_Read_PPU_DATA:
	lda #$00000100	; Disable NMI & Set Increment Per 1
	sta PPU_CTRL	; = $2000

	lda #$00000110	; Screen Display OFF
	sta PPU_MASK	; = $2001

	lda #$3F
	sta $2006
	lda #$00
	sta $2006

	lda $2007
	sta $00
	lda $2007
	sta $01
	
	:_SetPPUModeFirstDisplay:
	lda Data_PPU_CTRL	; set to execute NMI on Vblank
	sta PPU_CTRL		; set Control Register $2000			
	lda Data_PPU_MASK	; set to display screen		
	sta PPU_MASK		; set Control Register $2001	

	:_Fixed_Scrolling:
	ldx #$00
	stx PPU_SCROLL
	ldx #$00
	stx PPU_SCROLL

:_MAINLOOP												:
	

	:_@Wait_NMI:	
	lda VBlank_nCycles
	-			
	cmp VBlank_nCycles	;no Change in VBlank_nCycles : Loop to wait NMI	
	beq -
	
	nop

		
	jmp :_MAINLOOP:
	
:_NMI												:

	:_SaveRegisters:
	pha	; Push A on Stack
	txa	
	pha	; Push X on Stack
	tya
	pha	; Push Y on Stack
	
	
	:_Update_NameTable:
	lda Do_DrawBuffer
	beq @Done
	dec Do_DrawBuffer
	@Done

	
	:_Update_PPU_Registers:	 ; if Do_Update_PPU = 1
	lda Do_UpdatePPU
	beq @Done
	lda Data_PPU_CTRL
	sta PPU_CTRL				
	lda Data_PPU_MASK	
	sta PPU_MASK
	dec Do_UpdatePPU
	@Done	
	
	:_Update_Scrolling:	; if Do_Update_PPU = 2
	lda Do_UpdatePPU
	beq @Done
	ldx #$00
	stx PPU_SCROLL
	stx PPU_SCROLL
	dec Do_UpdatePPU
	@Done
		
	:_DeCount_Wait:
	lda Wait	
	beq @Done
	dec Wait
	@Done
	
	:_CountVBlankCycles:
	inc VBlank_nCycles	; from 0 to 255 one VblankTime ?
	
	:_RetrieveRegisters:
	pla	
	tay	; Pull Stack on Y
	pla	
	tax	; Pull Stack on X
	pla	; Pull Stack on A
	
rti




		
:_IRQ												:	
;IRQs may be generated by APU and by external hardware		
rti 	; Return from Interrupt

:_DATA												:
	
	::ClearCHRByte :
	.byte $99 ;(also debug byte)
	
:_PALETTE_DATA												:

	
	::Palette:
	
	; :Nametable:
	; .byte $08,$25,$36,$06
	HEX 08 25 36 06
	HEX 08 25 36 06
	HEX 08 25 36 06
	HEX 08 25 36 06


	; :Sprites:
	; .byte $08,$36,$3B,$07
	HEX 08 36 3B 07
	HEX 08 36 3B 07
	HEX 08 36 3B 07
	HEX 08 36 3B 07
	

:_INTERRUPTS												:
.org $FFFA      ; where starts vectors	
.word _NMI	; $FFFA -> $FFFB  (2bytes)
.word _RESET	; $FFFC -> $FFFD
.word _IRQ	; $FFFE -> $FFFF

:_CHR_BAKNS												:
.incbin "Table32ko.chr"