;---------------------------------------------------------------------------------------
; NOTICE REGISTERS									
;---------------------------------------------------------------------------------------

;Register 0 at $8000 : Control Register
;---------
;RxxC PSMM
;|  | ||||
;|  | ||++-------$MM Mirroring type :
;|  | ||	 $00 1 screen Mirroring with Nametable 0
;|  | ||	 $01 1 screen Mirroring with Nametable 1
;|  | ||	$10 Mirroring = Vertical
;|  | ||	$11 Mirroring = Horizontal
;|  | ||
;|  | |+--------$S Start PRG Address Bank
;|  | |		$0 Start Address at $8000
;|  | |		$1 Start Address at $C000 (POWER-ON)
;|  | |
;|  | +---------$P PRG Bank Size
;|  |		  $0 PRG Bank Size = 32K
;|  |		  $1 PRG Bank Size = 16K
;|  |
;|  +-----------$C CHR Bank Size
;|		$0 CHR Bank Size = 1 x 8K
;|		$1 CHR Bank Size = 2 x 4K
;|
;+--------------$R Reset bit
;		$0 Nothing
;		$1 RESET the shift register (must do typically on POWER-ON)

;Register 1 at $A000 : CHR Bank 0 >> Select the n CHR ROM Bank at $0000
;Register 2 at $C000 : CHR Bank 1 >> Select the n CHR ROM Bank at $1000
;Register 3 at $E000 : PRG Bank   >> Swapped ROM bank

;---------------------------------------------------------------------------------------
;Initialisation of our MAPPER MMC1							
;---------------------------------------------------------------------------------------
ResetMapper:
	lda #%10000000		; Reset bit = 1 
	sta $8000		; Write to Control Register
		
LoadControlRegister:		
	lda #%00010010		; Horizontal Mirroring, 32k PRG Bank Size start at $8000, 8k CHR Bank Size
	jsr SetControlRegister		

LoadCHRRegisterBank0:
	lda #$00	
	jsr SetCHRRegisterBank0

LoadCHRRegisterBank1:
	lda #$01		
	jsr SetCHRRegisterBank1

LoadPRGRegisterBank:
	lda #$00
	jsr SetPRGRegisterBank
	
jmp MapperReseted

;---------------------------------------------------------------------------------------
; Subroutines										
;---------------------------------------------------------------------------------------

SetControlRegister:
	sta $8000
	lsr A
	sta $8000
	lsr A
	sta $8000
	lsr A
	sta $8000
	lsr A
	sta $8000
	rts

SetCHRRegisterBank0:
	sta $A000
	lsr A
	sta $A000
	lsr A
	sta $A000
	lsr A
	sta $A000
	lsr A
	sta $A000
	rts

SetCHRRegisterBank1:
	sta $C000
	lsr A
	sta $C000
	lsr A
	sta $C000
	lsr A
	sta $C000
	lsr A
	sta $C000
	rts

SetPRGRegisterBank:
	sta $E000
	lsr A
	sta $E000
	lsr A
	sta $E000
	lsr A
	sta $E000
	lsr A
	sta $E000
	rts
	
MapperReseted: